#!/usr/bin/env python2
# -*- coding: utf-8 -*-

import numpy as np
import matplotlib.pyplot as plt

plt.rc('text', usetex=True, fontsize=8)
plt.rc('font', family='serif')

def plot_registro_temporal(df, yy_zoom, df_vars_ciclos, umbral):
    """Funcion que proporciona la grafica de la serie temporal de ciclos
        
        Args:
            
            df: dataframe de registro original
            yy_zoom: años que se desean mostrar
            df_vars_ciclos: dataframe con informacion de los ciclos
            umbral: umbral de calculo inicial
        
        Returns:
            grafico de serie temporal de ciclos    
    """

    df_zoom = df[df.index.year == yy_zoom]
    df_vars_ciclos_zoom = df_vars_ciclos[df_vars_ciclos.index.year == yy_zoom]

    df_zoom2 = df_zoom.loc[(df_zoom.index.month == 10) | (df_zoom.index.month == 11) |
                           (df_zoom.index.month == 12)]
    df_vars_ciclos_zoom2 = df_vars_ciclos_zoom.loc[(df_vars_ciclos_zoom.index.month == 10) |
                                                   (df_vars_ciclos_zoom.index.month == 11) |
                                                   (df_vars_ciclos_zoom.index.month == 12)]

    fs, axs = plt.subplots(3, 2, figsize=(10, 8))
    axs[0, 0].plot(df_zoom.index, df_zoom.hs, 'blue')
    axs[0, 0].plot(df_zoom.index, np.ones(len(df_zoom.hs)) * umbral, 'k--')
    axs[0, 0].plot(df_vars_ciclos_zoom.index, df_vars_ciclos_zoom.hs_max, 'ro')
    axs[0, 0].set_ylabel(r'$H_s$ $(m)$')
    plt.setp(axs[0, 0].get_xticklabels(), rotation=45)

    axs[0, 1].plot(df_zoom2.index, df_zoom2.hs, 'blue')
    axs[0, 1].plot(df_zoom2.index, np.ones(len(df_zoom2.hs)) * umbral, '--')
    axs[0, 1].plot(df_vars_ciclos_zoom2.index, df_vars_ciclos_zoom2.hs_max, 'ro')
    plt.setp(axs[0, 1].get_xticklabels(), rotation=45)

    axs[1, 0].plot(df_zoom.index, df_zoom.tp, 'blue')
    axs[1, 0].set_ylabel(r'$T_p$ $(m)$')
    plt.setp(axs[1, 0].get_xticklabels(), rotation=45)

    axs[1, 1].plot(df_zoom2.index, df_zoom2.tp, 'blue')
    plt.setp(axs[1, 1].get_xticklabels(), rotation=45)

    axs[2, 0].plot(df_zoom.index, df_zoom.dh, 'blue')
    axs[2, 0].set_ylabel(r'$\theta$ $(m)$')
    plt.setp(axs[2, 0].get_xticklabels(), rotation=45)

    axs[2, 1].plot(df_zoom2.index, df_zoom2.dh, 'blue')
    plt.setp(axs[2, 1].get_xticklabels(), rotation=45)

    plt.tight_layout(pad=0.2, w_pad=0.1, h_pad=0.4)


def plot_dispersion(df_vars_ciclos, var_x, var_y, name_x, name_y):
    """Funcion que proporciona la grafica de dispersion de los datos

       Args:

           df_vars_ciclos: dataframe con informacion de los ciclos
           
       Returns:
           grafico de dispersion de los datos   
       """

    plt.figure()
    plt.plot(df_vars_ciclos[var_x], df_vars_ciclos[var_y], '.')
    plt.xlabel(name_x)
    plt.ylabel(name_y)


def plot_dispersion_simul(df_vars_ciclos, var_x, var_y, name_x, name_y, df_sim_x, df_sim_y):
    """Funcion que proporciona la grafica de dispersion de los datos

       Args:

           df_vars_ciclos: dataframe con informacion de los ciclos

       Returns:
           grafico de dispersion de los datos   
       """

    plt.figure()
    plt.plot(df_vars_ciclos[var_x], df_vars_ciclos[var_y], '.k', label=r'$Registro$')
    plt.plot(df_sim_x, df_sim_y, '.b', alpha=0.5, label=r'$Simulado$')
    plt.xlabel(name_x)
    plt.ylabel(name_y)
    plt.legend()


def plot_simulacion_ciclos(df_sim, vars, nvars, fechai, fechaf):
    """Funcion que proporciona la grafica los resultados de la simulacion de ciclos independientes

           Args:

               df_sim: dataframe con informacion de los ciclos simulados
               vars: variables a analizar
               nvars: etiqueta de las variables
               fechai: fecha de inicio a representar
               fechaf: fecha de fin a representar               

           Returns:
               grafico de dispersion de los datos   
           """

    plt.figure()
    ax = plt.subplot(3, 1, 1)
    plt.plot(df_sim[vars[0]][fechai:fechaf])
    plt.setp(ax.get_xticklabels(), visible=False)
    plt.ylabel(nvars[0])
    ax2 = plt.subplot(3, 1, 2, sharex=ax)
    plt.plot(df_sim[vars[1]][fechai:fechaf])
    plt.setp(ax2.get_xticklabels(), visible=False)
    plt.ylabel(nvars[1])
    plt.subplot(3, 1, 3, sharex=ax)
    plt.plot(df_sim[vars[2]][fechai:fechaf])
    plt.xticks(rotation=30)
    plt.subplots_adjust(bottom=0.25)
    plt.ylabel(nvars[2])


def plot_copulas_estacion(df_pri, xd_pri, yd_pri, df_ver, xd_ver, yd_ver, df_oto, xd_oto, yd_oto, df_inv, xd_inv, yd_inv, var_x, var_y, lab_x, lab_y):

    fc, axsc = plt.subplots(1, 4, figsize=(5.3,1.8), sharey=True)
    axsc[0].plot(df_pri[var_x], df_pri[var_y], '.', label=r'$Data$')
    axsc[0].plot(xd_pri, yd_pri, '.k', alpha=0.5, label=r'$Simulation$')
    axsc[0].set_ylabel(lab_y)
    # axsc[0].set_title(r'$Spring$', fontsize=8)
    # axsc[0].set_xlabel(lab_x)

    axsc[1].plot(df_ver[var_x], df_ver[var_y], '.')
    axsc[1].plot(xd_ver, yd_ver, '.k', alpha=0.5)
    # axsc[1].set_title(r'$Summer$', fontsize=8)
    # axsc[1].set_xlabel(lab_x)

    axsc[2].plot(df_oto[var_x], df_oto[var_y], '.')
    axsc[2].plot(xd_oto, yd_oto, '.k', alpha=0.5)
    # axsc[2].set_xlabel(lab_x)
    # axsc[2].set_title(r'$Fall$', fontsize=8)

    axsc[3].plot(df_inv[var_x], df_inv[var_y], '.')
    axsc[3].plot(xd_inv, yd_inv, '.k', alpha=0.5)
    # axsc[3].set_xlabel(lab_x)
    # axsc[3].set_title('$Winter$', fontsize=8)

    return fc, axsc


def plot_registro_temporal_entero(df, df_vars_ciclos, umbral, name, zoom=None):
    """Funcion que proporciona la grafica de la serie temporal de ciclos

        Args:

            df: dataframe de registro original
            df_vars_ciclos: dataframe con informacion de los ciclos
            umbral: umbral de calculo inicial

        Returns:
            grafico de serie temporal de ciclos    
    """

    if zoom:
        df_zoom = df[df.index.year == zoom]
        df_vars_ciclos_zoom = df_vars_ciclos[df_vars_ciclos.index.year == zoom]
    else:
        df_zoom = df
        df_vars_ciclos_zoom = df_vars_ciclos

    fs, axs = plt.subplots(3, 1, figsize=(2.7, 2), sharex=True)
    axs[0].plot(df_zoom.index, np.ones(len(df_zoom.hs)) * umbral, color='k', linestyle='--',
                label=r'$H_{s,u}$ =$' + '{:.2f}'.format(umbral) + '$ $ m$', linewidth=0.6)
    axs[0].legend(bbox_to_anchor=(0.5, 1.1), loc='upper center', frameon=False)
    axs[0].plot(df_zoom.index, df_zoom.hs, linewidth=0.6)
    axs[0].plot(df_vars_ciclos_zoom.index, df_vars_ciclos_zoom.hs_max, 'o', color='gray', markersize=2)
    axs[0].set_ylabel(r'$H_s$ $(m)$')

    # axs[0].text(df_zoom.index.min(), umbral, r'$H_u=$' + str(umbral) + '$m$')
    # plt.setp(axs[0].get_xticklabels(), rotation=45)

    axs[1].plot(df_zoom.index, df_zoom.tp, linewidth=0.6)
    axs[1].set_ylabel(r'$T_p$ $(s)$')
    # plt.setp(axs[1].get_xticklabels(), rotation=45)

    axs[2].plot(df_zoom.index, df_zoom.dh, linewidth=0.6)
    axs[2].set_ylabel(r'$\theta$ $(^\circ)$')
    # plt.setp(axs[2].get_xticklabels(), rotation=45)

    plt.xticks(rotation=13)
    plt.tight_layout(pad=0.2, w_pad=0.1, h_pad=0.4)
    plt.suptitle(name, y=1.03)

    return fs, axs