# -*- coding: utf-8 -*-
import numpy as np
import os


def cabecera(COD):
    """Funcion que genera la cabecera del fichero SIMAR

    Args:
        COD: Codigo del fichero SIMAR

    Returns:
        La cabecera del fichero SIMAR

    """

    cabec = [' C01 CONJUNTO DE DATOS SIMULADOS A PARTIR LOS DATOS WANA\n'
             ' C02\n'
             ' C03 CODIGO PUNTO WANA DE LOS DATOS ORIGINALES ' +
             str(COD) + '\n'
             ' C04\n'
             ' C05 Datos simulados utilizando distribuciones de probabilidad'
             ' no estacionarias\n'
             ' C06 y modelos autoregresivos vectoriales.\n'
             ' C07 Los parametros de los modelos fueron ajustados utilizando'
             ' los datos WANA,\n'
             ' C08 por lo que las series simuladas conservan todas las'
             ' limitaciones\n'
             ' C09 de dichos datos, y se suman las propias del metodo de'
             ' simulacion.\n'
             ' C10 Para conocer las limitantes del metodo de simulacion ver'
             ' documentacion\n'
             ' C11 complementaria.\n'
             ' C12\n'
             ' C13 A continuacion se incluye la informacion contenida de'
             ' forma\n'
             ' C14 en los archivos de datos WANA.\n'
             ' C15\n'
             ' C16 PROCEDENCIA DE LOS DATOS\n'
             ' C17\n'
             ' C18 Datos sinteticos obtenidos a partir de modelos numericos\n'
             ' C19 HIRLAM Y WAM operativos en el Instituto Nacional de'
             ' Meteorologia\n'
             ' C20\n'
             ' C21 El modelo HIRLAM es un modelo atmosferico de tipo'
             ' mesoescalar\n'
             ' C22 la resolucion espacial media es de 15 Km. Los datos'
             ' tienen\n'
             ' C23 una cadencia de 3 horas, pero representan valores medios\n'
             ' C24 horarios. Los datos de viento se encuentran reducidos a'
             ' 10 m\n'
             ' C25 de altura\n'
             ' C26\n'
             ' C27 El molelo WAN es un modelo de generacion de oleaje que'
             ' trabaja\n'
             ' C28 con una resolucion media de 30 Km en el Atlantico y 15 Km'
             ' en\n'
             ' C29 el Mediterraneo.\n'
             ' C30\n'
             ' C31 PRECAUCIONES\n'
             ' C32\n'
             ' C33 Los datos de oleaje proporcionan, siempre, condicioens en'
             ' aguas\n'
             ' C34 abiertas y profundidades indefinidas\n'
             ' C35\n'
             ' C36 Los datos de viento no reproducen efectos geograficos de'
             ' baja\n'
             ' C37 escala, ni procesos con ecala temporal inferior a 6 horas.'
             '\n'
             ' C38 De modo general son mas fiables los vientos procedentes de'
             ' mar\n'
             ' C39\n'
             ' C40 El conjunto de datos WANA proporciona descripciones'
             ' adecuadas\n'
             ' C41 en casi todas las zonas. No obstante es necesario tener'
             ' cautela\n'
             ' C42 en las siguientes zonas:\n'
             ' C43\n'
             ' C44   - En el Estrecho de Gibraltar no se reproducen bien los\n'
             ' C45     oleajes propagados de una cuenca a otra\n'
             ' C46\n'
             ' C47   - En el Norte de la Costa Catalana y Golfo de Leon'
             ' pueden\n'
             ' C48     sobreestimarse las velocidades de viento y las alturas'
             ' de\n'
             ' C49     ola en situaciones de temporal muy extremo.\n'
             ' C50     Se aconseja no eliminar los datos sobreestimados,'
             ' sino\n'
             ' C51     mas bien cotejar la magnitud aproximada del temporal'
             ' con\n'
             ' C52     datos instrumentales de la zona.\n'
             ' C53\n'
             ' C54   - En el Sur del Archipielago Canario pueden no'
             ' reproducirse\n'
             ' C55     bien condiciones procedentes del SurOeste\n'
             ' C56\n'
             ' C57 LISTA DE PARAMETROS\n'
             ' C58\n'
             ' C59 Hm0    : Altura significante Espectral                  '
             '(m)\n'
             ' C60 Tm02   : Periodo Medio Espectral Momentos 0 y 2         '
             '(NO INCLUIDO)\n'
             ' C61 Tp     : Periodo de pico espectral                      '
             '(s)\n'
             ' C62 DirM   : Direccion Media de PROCEDENCIA del Oleaje      '
             '(0=N,90=E)\n'
             ' C63\n'
             ' C64 Hm0_V  : Mar de viento: Altura Signifcante Espectral    '
             '(NO INCLUIDO)\n'
             ' C65 DirM_V : Mar de Viento: Direccion Media de PROCEDENCIA  '
             '(NO INCLUIDO)\n'
             ' C66\n'
             ' C67 Hm0_F1 : Mar de Fondo 1: Altura Significante Espectral  '
             '(NO INCLUIDO)\n'
             ' C68 Tm02_F1: Mar de Fondo 1: Periodo Medio Espectral        '
             '(NO INCLUIDO)\n'
             ' C69 DirM_F1: Mar de Fondo 1: Direccion Media de PROCEDENCIA '
             '(NO INCLUIDO)\n'
             ' C70\n'
             ' C71 Hm0_F2 : Mar de Fondo 2: Hm0                            '
             '(NO INCLUIDO)\n'
             ' C72 Tm02_F2: Mar de Fondo 2: Periodo Medio Espectral        '
             '(NO INCLUIDO)\n'
             ' C73 DirM_F2: Mar de Fondo 2: Direccion Media de PROCEDENCIA '
             '(NO INCLUIDO)\n'
             ' C74\n'
             ' C75 VelV   : Velocidad Media del Viento                     '
             '(m/s)\n'
             ' C76 DirV   : Direccion Media de PROCEDENCIA del Viento      '
             '(0=N,90=E)\n'
             ' C77\n'
             ' C78\n'
             ' C79 LISTADO DE DATOS\n'
             ' C80\n'
             ' C81 AA MM DD HH      Hm0   Tm02   Tp   DirM      Hm0_V  DirM_V '
             'Hm0_F1 Tm02_F1 DirM_F1 Hm0_F2 Tm02_F2 DirM_F2   VelV   DirV']

    return cabec


def escribe_simar(ruta, codsimar, sim):
    """Funcion que escribe el Simar simulado en formato SIMAR (Puertos del
    Estado)

    Args:
        ruta: Directorio (ruta absoluta) del fichero de salida
        codsimar: Codigo del Simar
        simarsim: Matriz con los datos simulados

    Return:
        Escribe el fichero denominado ``SIMAR_codSimar_SIM``

    """

    cabec = cabecera(codsimar)
    np.savetxt(os.path.join(ruta, 'SIMAR_'+str(codsimar)+'_SIM'), sim,
               fmt=('%9d %2d %2d %2d %8.1f %6.1f %4.1f %6.1f %10.1f %7.1f'
                    '%6.1f %7.1f %7.1f %6.0f %7.0f %7.0f %6.1f %6.1f'),
               newline='\n', header=cabec[0])


def escribe_fichero(ruta, nombre, sim):
    """Funcion que escribe el resultado de la simulacion

    Args:
        ruta: Directorio (ruta absoluta) del fichero de salida
        nombre: Nombre del fichero
        sim: Matriz con los datos simulados

    Return:
        Escribe el fichero denominado ``nombre_sim``

    """
    formato = '%4i %2i %2i %2i %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f'
    np.savetxt(os.path.join(ruta, nombre+'_sim'), sim, fmt=formato)
