# -*- coding: utf-8 -*-
import matplotlib.pyplot as plt
import numpy as np
from scipy.stats import lognorm, norm, genextreme
from matplotlib import gridspec

def dscatter(x, y, *varargin):
    """

    """
    lambda_ = 20

    minx, maxx, miny, maxy = np.min(x), np.max(x), np.min(y), np.max(y)

    nbins = np.array([np.min((np.size(np.unique(x)), 200)), np.min((np.size(np.unique(y)), 200))])

    edges1 = np.linspace(minx, maxx, nbins[0]+1)
    edges1 = np.hstack((-np.inf, edges1[1:-1], np.inf))
    edges2 = np.linspace(miny, maxy, nbins[0]+1)
    edges2 = np.hstack((-np.inf, edges2[1:-1], np.inf))

    n = nbins[0]+1
    bin_ = np.zeros([n, 2])
    # Reverse the columns to put the first column of X along the horizontal
    # axis, the second along the vertical.
    dum, bin_[:, 1] = np.histogram(x, bins=edges1)
    dum, bin_[:, 0] = np.histogram(y, bins=edges2)
    h = np.bincount(bin_, weights=nbins([1, 0]))/n
    g = smooth1D(h, nbins[1]/lambda_)
    f = smooth1D(g.T, nbins[0]/lambda_).T

    f = f/np.max(f)
    ind = np.ravel_multi_index(np.shape(f), bin_[:, 0], bin[:, 1], order='F')
    col = f[ind]
    h = plt.scatter(x, y, col, 's', 'filled')


def smooth1D(y, lambda_):
    m, n = np.shape(y)
    e = np.eye(m)
    d1 = np.diff(e)
    d2 = np.diff(d1)
    p = lambda_**2*d2.T*d2+2*lambda_*d1.T*d1
    z = (e + p)/y
    return z


def condicional(xpar, xparaux, xlim, ylim, alpha, dist, param, yest, reg, xg, yg, pdfg, ci, x, y, xlab, ylab, yrlab, valmed):
    """ Funcion que dibuja los resultados del regimen conjunto a partir de la funcion condicional

    Args:
        - xpar:
        - xlim:
        - ylim:
        - alpha:
        - dist:
        - param:
        - yest:
        - reg:
        - xg:
        - yg:
        - pdfg:
        - ci:
        - x:
        - y:

    Returns:
        La grafica

    """

    xpar2 = np.asarray(xparaux)

    ndat = len(yest)

    ndat_max = 10000
    if ndat > ndat_max:
        # ids_r = np.random.choice(len(yest), ndat_max, replace=False)
        ids_r = np.linspace(0, ndat-1, ndat_max, dtype=int)
        xpar = xpar[ids_r, :]
        xpar2 = xpar2[ids_r, :]
        yest = yest[ids_r, :]
        x = x[ids_r]
        y = y[ids_r]

    plt.style.use('ggplot')
    xaux = np.linspace(np.min(x), np.max(x), 1000)

    fig = plt.figure()
    gs = gridspec.GridSpec(2, 6)

    ax1 = fig.add_subplot(gs[0, 0:2])
    id1 = np.where(xpar[:, -2] >= alpha)
    id2 = np.where(xpar[:, -2] < alpha)
    ax1.plot(xpar[id1, 0], xpar[id1, -2], 'b.')
    ax1.plot(xpar[id2, 0], xpar[id2, -2], 'r.')
    ax1.plot((np.min(xpar[:, 0]), np.max(xpar[:, 0])), (alpha, alpha), '-r')
    # ax1.set_ylim([0,1])
    ax1.set_xlabel(xlab)
    ax1.set_ylabel('p-valor')
    ax1.tick_params(axis='x', labelsize=10)
#    plt.legend((r'$p-valor > \alpha$', r'$p-valor < \alpha$', r'$\alpha$'), loc='best', )
#     plt.title('chi2test - ajuste con dist. '+dist)

    npar = len(xparaux[0])


    if npar == 2:
        ini = 0
        for i in range(npar):
            ax2 = fig.add_subplot(gs[1, ini:ini + 3])
            ax2.plot(xpar[:, 0], xpar2[:, i], '.', color='gray')
            ax2.plot(xaux, param[i], '-r', lw=2)
            ax2.set_xlabel(xlab)
            ax2.set_ylabel(dist.name + [' - localizacion', ' - escala'][i])
            ax2.tick_params(axis='x', labelsize=8)
            ini += 3

    elif npar == 3:
        ini = 0
        for i in range(npar):
            ax2 = fig.add_subplot(gs[1, ini:ini + 2])
            ax2.plot(xpar[:, 0], xpar2[:, i], '.', color='gray')
            ax2.plot(xaux, param[i], '-r', lw=2)
            ax2.set_xlabel(xlab)
            ax2.set_ylabel(dist.name + [' - forma', ' - localizacion', ' - escala'][i])
            ax2.tick_params(axis='x', labelsize=8)
            ini += 2

    ax3 = fig.add_subplot(gs[0, 2:-2])
    id_ = np.where(np.isnan(yest) | np.isinf(yest))[0]
    if any(id_):
        print 'Hay valores reconstruidos de la variable que son nan o inf'
    id_ = np.where(~np.isnan(yest) | ~np.isinf(yest))[0]
#    dscatter(xpar[id_, 1], yest[id_])
    ax3.scatter(xpar[id_, 1], yest[id_])
    ax3.plot(xpar[id_, 1], xpar[id_, 1], 'k', lw=2)
    ax3.set_xlabel(ylab)
    ax3.set_ylabel(yrlab)
    ax3.tick_params(axis='x', labelsize=8)
    # plt.title('Y original - Y reconstruida')

    ax4 = fig.add_subplot(gs[0, 4:])
#    dscatter(x, y)
    ax4.contour(xg, yg, pdfg, 6)
    ax4.scatter(x, y, 10, 'gray', label='datos')
    ax4.plot(xaux, valmed, 'k', lw=2, label='ajuste')
    ax4.plot(xaux, ci[0, :], '--r', lw=2, label='ic')
    ax4.plot(xaux, ci[1, :], '--r', lw=2)
    ax4.legend(loc='best', scatterpoints=1, fontsize=8)
    ax4.set_xlim([np.min(x), np.max(x)])
    # plt.tight_layout(pad=0.5)
    ax4.set_xlabel(xlab)
    ax4.set_ylabel(ylab)
    ax4.tick_params(axis='x', labelsize=8)


    plt.tight_layout(pad=0.2, w_pad=0.1, h_pad=0.4)


def plot_copula(x, y, cop, x_ap, y_ap, cop_ap, nombrex, nombrey):
    """ Dibuja el resultado del ajuste de la copula de Fourier

    Args:
        - x: datos de la variable principal
        - y: datos de la variable secundaria
        - cop: copula empirica
        - cop_ap: copula ajustada
        - nombrex: nombre de la variable principal
        - nombrey: nombre de la variables secundaria

    Returns:
        Dibuja la figura

    """

    plt.style.use('ggplot')
    plt.figure(figsize=(4, 4))
    plt.contour(x_ap, y_ap, cop_ap, levels=np.arange(0.1, 0.99, 0.1), colors='k', label='Copula de Fourier')
    plt.contour(x, y, cop, levels=np.arange(0.1, 0.99, 0.1), colors='b', linestyles='dotted', label='Copula empírica')
    plt.xlabel(nombrex)
    plt.ylabel(nombrey)
    plt.legend(loc='best')
    plt.gcf().subplots_adjust(bottom=0.2, left=0.2)
