# -*- coding: utf-8 -*-
"""
Created on Fri Jun 17 14:51:46 2016

@author: usuario
"""
from __future__ import division
import numpy as np
import pandas as pd


def cossen_n(mod, n):
    """ Funcion que calcula la evolucion de los parametros de Fourier a lo largo del anio

    Args:
        - mod: orden maximo de la expansion
        - n: vector con la fecha en formato [0 - 1]

    Returns:
        Devuelve una matriz con variacion de las diferentes componentes

    """

    cossen = np.zeros([np.max(mod)*2, len(n)])
    w = 2*np.pi
    if np.max(mod) != 0:
        for i in range(0, np.max(mod)):
            aux = (i+1)*w*n.T
            cossen[2*i, :] = np.cos(aux)
            cossen[2*i+1, :] = np.sin(aux)

    return cossen


def picos_dur(data, dur):
    """ Funcion que busca los eventos maximos separados al menos por una duracion 'dur/2' y devuelve sus indices.

    Args:
        - data: vector de la variable
        - dur: duracion

    Returns:
        Devuelve un vector con los indices de la variables principal donde se encuentran los picos

    """
    dur = int(dur)
    n = len(data)
    picos = list()
    for k in range(0, n-dur+1):
        id_ = np.nanargmax(data.iloc[k:k+dur+1])
        if id_ == int(dur/2):
            picos.append(k + int(dur/2))

    picos = np.asarray(picos, dtype=np.int)
    return picos


def picos_umb(picos, umb):
    """ Funcion que busca los picos que superan un determinado umbral.

    Args:
        - picos: serie con los picos
        - umb: umbral minimo

    Returns:
        Devuelve una lista con los indices de los picos que superan el umbral.

    """

    picumbid = list()
    for i in range(0, len(umb)):
        id_ = np.where(picos > umb[i])[0]
        picumbid.append(id_)

    return picumbid


def maximos_anuales(df):
    """ Funcion que busca los eventos maximos anuales y devuelve el dataframe correspondiente a éstos.

    Args:
        - df:

    Returns:

    """

    max_anu_ref = df.groupby(df.index.year).max()

    df_maxanu = pd.DataFrame()
    for index_max, row_max in max_anu_ref.iterrows():
        df_anio_i = df[df.index.year == index_max]
        df_max_anual_i = df_anio_i[df_anio_i.iloc[:, 0] == row_max.values[0]]
        df_maxanu = df_maxanu.append(df_max_anual_i)

    return df_maxanu


def picos_ind(p1, p2, ven):
    """ Funcion que busca los eventos maximos separados al menos por una duracion 'dur/2' y devuelve sus indices.

    Args:
        - p1: indices de los picos de la variable 1
        - p2: indices de los picos de la variable 2
        - ven: ventana temporal en indices

    Returns:
        Devuelve un vector con los indices independientes y el numero de ellos

    """

    p2_i = np.array([])
    for p2i in p2:
        if (np.abs(p1 - p2i) >= ven/2).all():
            np.append(p2_i, p2i)

    npicos_ind = len(p1) + len(p2_i)
    return p2_i, npicos_ind
