# -*- coding: utf-8 -*-
import re
import numpy as np
from datetime import datetime
import os


def lee_simar(ruta, SIMAR):
    """Funcion que lee el fichero ``SIMAR``.

    Args:
        ruta: Directorio (ruta absoluta) del fichero de entrada
        SIMAR: fichero del SIMAR.

    Returns:
        Un tupla de la forma (COD,NORIG,H,T,D,VV,DV,N).

        * ``COD``: Codigo del fichero SIMAR.
        * ``NORIG``: Fecha en ano/mes/dia/hora.
        * ``H``: Altura de ola significante.
        * ``T``: Periodo de pico espectral.
        * ``D``: Direccion media del oleaje.
        * ``VV``: Velocidad de viento.
        * ``DV``: Direccion de viento.
        * ``N``: Fecha en formato numerico.

    """

    data = list()
    iddata = 0
    fid = open(os.path.join(ruta, SIMAR), 'r')
    COD = 0

    for line in fid:
            if re.search('CODIGO WANA\s+ [0-9]+', line):
                COD = (re.findall('[0-9]+', line))[0]
            if re.search('(\d{4}\s\d{2}\s\d{2}\s\d{2}'
                         '[\s*-?\d+[\.\d+]*]*)', line):
                break
            iddata += 1
    fid.close()

    data = np.loadtxt(os.path.join(ruta, SIMAR), skiprows=iddata)
    NORIG = data[:, 0:4]
    H = data[:, 4]
    T = data[:, 6]
    D = data[:, 7]
    VV = data[:, 16]
    DV = data[:, 17]

    N = np.zeros(len(NORIG))
    for i in range(0, len(NORIG)):
        N[i] = datetime(int(NORIG[i, 0]), int(NORIG[i, 1]),
                        int(NORIG[i, 2])).toordinal() + NORIG[i, 3]/24.

    return COD, NORIG, H, T, D, VV, DV, N
