# -*- coding: utf-8 -*-
"""
Created on Thu Apr 14 10:14:35 2016
"""
from __future__ import division
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import dates
from scipy.stats import norm
from clima_maritimo.clima_maritimo.fdist import ajuste_simple as ajs
# from clima_maritimo.ttide.t_tide import t_predic # para ejecutar desde dentro de clima_maritimo
import time
from clima_maritimo.clima_maritimo.ttide.t_tide import t_predic # para ejecutar desde fuera de clima_maritimo
import windrose
from SNL import FDS

def cdf_pdf_registro(x, name):
    """ Dibuja las funciones de distribucion acumuladas de las variables

    Args:
        X: vector con la variable del registro SIMAR
        XS: vector con la variable del registro simulado

    Returns:
        Las graficas comparativas de la cdf y pdf

    """

    plt.style.use('ggplot')

    fig, ax1 = plt.subplots(figsize=(4, 4))
    yp, xp = np.histogram(x, bins=25, normed=True)
    xp = np.diff(xp) / 2 + xp[0:-1]
    plt.plot(xp, yp, 'gray')
    plt.xlabel(name, fontweight='bold')
    plt.ylabel('pdf', fontweight='bold')

    Yp = np.cumsum(yp)/np.cumsum(yp)[-1]
    plt.plot(xp, Yp, 'gray', label='Registro')
    plt.ylabel('cdf', fontweight='bold')
    plt.legend(loc='best', numpoints=1)
    plt.gcf().subplots_adjust(bottom=0.2, left=0.15, right=0.85)

    fig2 = plt.figure(figsize=(4, 4))
    plt.plot(xp, norm.ppf(Yp), 'gray', label='Registro')

    ytick = [.001, .01, .1, .25, .5, .75, .9, .99, .999]
    ytickp = norm.ppf(ytick)
    plt.yticks(ytickp, ytick)
    plt.xlabel(name, fontweight='bold')
    plt.ylabel('Z', fontweight='bold')
    plt.legend(loc='best', numpoints=1)
    plt.gcf().subplots_adjust(bottom=0.2, left=0.2)

    fig.savefig('cdf_pdf_' + name + '.png')
    fig2.savefig('cdf_q_' + name + '.png')

    return


def cdf_pdf(x, xs, name):
    """ Dibuja las funciones de distribucion acumuladas de las variables

    Args:
        X: vector con la variable del registro SIMAR
        XS: vector con la variable del registro simulado

    Returns:
        Las graficas comparativas de la cdf y pdf

    """

    plt.style.use('ggplot')

    plt.figure()
    plt.subplot(1, 2, 1)
    yp, xp = np.histogram(x, bins=25, normed=True)
    ysp, xsp = np.histogram(xs, bins=25, normed=True)
    xp, xsp = np.diff(xp)/2 + xp[0:-1], np.diff(xsp)/2 + xsp[0:-1]
    plt.plot(xp, yp, 'gray')
    plt.plot(xsp, ysp, 'k')
    plt.xlabel(name, fontweight='bold')
    plt.ylabel('pdf', fontweight='bold')

    plt.subplot(1, 2, 2)
    Yp = np.cumsum(yp)/np.cumsum(yp)[-1]
    Yps = np.cumsum(ysp)/np.cumsum(ysp)[-1]
    plt.plot(xp, Yp, 'gray', label='Registro')
    plt.plot(xsp, Yps, 'k', label='Simulacion')
    plt.ylabel('cdf', fontweight='bold')
    plt.legend(loc='best', numpoints=1)
    plt.xlabel(name, fontweight='bold')
    plt.gcf().subplots_adjust(bottom=0.2, left=0.15, right=0.85)

    plt.figure()
    plt.plot(xp, norm.ppf(Yp), 'gray', label='Registro')
    plt.plot(xsp, norm.ppf(Yps), 'k', label='Simulacion')

    ytick = [.001, .01, .1, .25, .5, .75, .9, .99, .999]
    ytickp = norm.ppf(ytick)
    plt.yticks(ytickp, ytick)
    plt.xlabel(name, fontweight='bold')
    plt.ylabel('CDF', fontweight='bold')
    plt.legend(loc='best', numpoints=1)
    plt.gcf().subplots_adjust(bottom=0.2, left=0.2)

    return

def cdf_sim(x, xs, name, axis):
    """ Dibuja las funciones de distribucion acumuladas de las variables

    Args:
        X: vector con la variable del registro SIMAR
        XS: vector con la variable del registro simulado

    Returns:
        Las graficas comparativas de la cdf y pdf

    """

    yp, xp = np.histogram(x, bins='auto', normed=True)
    ysp, xsp = np.histogram(xs, bins='auto', normed=True)
    xp, xsp = np.diff(xp)/2 + xp[0:-1], np.diff(xsp)/2 + xsp[0:-1]

    Yp = np.cumsum(yp) / np.cumsum(yp)[-1]
    Yps = np.cumsum(ysp) / np.cumsum(ysp)[-1]

    axis.plot(xp, Yp, 'gray', label='Data')
    axis.plot(xsp, Yps, 'k', label='Fit')

    axis.set_xlabel(name)
    axis.grid(True)
    axis.set_ylabel(r'$CDF$')


    return


def cdf_pdf_perc(x, xs, name, axis):
    """ Dibuja las funciones de distribucion acumuladas de las variables

    Args:
        X: vector con la variable del registro SIMAR
        XS: vector con la variable del registro simulado

    Returns:
        Las graficas comparativas de la cdf y pdf

    """

    yp, xp = np.histogram(x, bins='auto', normed=True)
    ysp, xsp = np.histogram(xs, bins='auto', normed=True)
    xp, xsp = np.diff(xp)/2 + xp[0:-1], np.diff(xsp)/2 + xsp[0:-1]

    Yp = np.cumsum(yp)/np.cumsum(yp)[-1]
    Yps = np.cumsum(ysp)/np.cumsum(ysp)[-1]

    axis.plot(xp, norm.ppf(Yp), '.', label=r'$Data$', markersize=3)
    axis.plot(xsp, norm.ppf(Yps), 'gray', label=r'$Fit$')

    ytick = [.001, .01, .1, .25, .5, .75, .9, .99, .999]
    ytickp = norm.ppf(ytick)
    axis.set_yticks(ytickp)
    axis.set_yticklabels(ytick)
    axis.set_xlabel(name)
    axis.grid(True)

    return


def cdf_pdf_perc_list(x, xslist, name, axis):
    """ Dibuja las funciones de distribucion acumuladas de las variables

    Args:
        X: vector con la variable del registro SIMAR
        XS: vector con la variable del registro simulado

    Returns:
        Las graficas comparativas de la cdf y pdf

    """


    for xs in xslist:
        ysp, xsp = np.histogram(xs, bins='auto', normed=True)
        xsp = np.diff(xsp)/2 + xsp[0:-1]
        Yps = np.cumsum(ysp)/np.cumsum(ysp)[-1]
        linesim, = axis.plot(xsp, norm.ppf(Yps), 'gray', alpha=0.7, label=r'$Simulation$')

    yp, xp = np.histogram(x, bins='auto', normed=True)
    xp = np.diff(xp) / 2 + xp[0:-1]
    Yp = np.cumsum(yp) / np.cumsum(yp)[-1]
    lineini, = axis.plot(xp, norm.ppf(Yp), '.', label=r'$Data$', markersize=3)

    ytick = [.001, .01, .1, .25, .5, .75, .9, .99, .999]
    ytickp = norm.ppf(ytick)
    axis.set_yticks(ytickp)
    axis.set_yticklabels(ytick)
    axis.set_xlabel(name)
    axis.grid(True)

    return lineini, linesim


def cdf_perc_pdf(x, xs, name, axis1, axis2):
    """ Dibuja las funciones de distribucion acumuladas de las variables

    Args:
        X: vector con la variable del registro SIMAR
        XS: vector con la variable del registro simulado

    Returns:
        Las graficas comparativas de la cdf y pdf

    """

    yp, xp = np.histogram(x, bins='auto', normed=True)
    ysp, xsp = np.histogram(xs, bins='auto', normed=True)
    xp, xsp = np.diff(xp)/2 + xp[0:-1], np.diff(xsp)/2 + xsp[0:-1]

    Yp = np.cumsum(yp)/np.cumsum(yp)[-1]
    Yps = np.cumsum(ysp)/np.cumsum(ysp)[-1]

    axis1.plot(xp, norm.ppf(Yp), label=r'$Data$')
    axis1.plot(xsp, norm.ppf(Yps), 'gray', label=r'$Fit$')

    ytick = [.001, .01, .1, .25, .5, .75, .9, .99, .999]
    ytickp = norm.ppf(ytick)
    axis1.set_yticks(ytickp)
    axis1.set_yticklabels(ytick)
    axis1.grid(True)

    axis2.hist((x, xs), normed=True, alpha=0.5, histtype='bar')
    # axis2.hist(xs, normed=True, alpha=0.5)

    # axis2.plot(xp, yp, '.')
    # axis2.plot(xsp, ysp, 'gray')
    axis2.grid(True)
    axis2.set_xlabel(name)

    return


def cdf_pdf_xslist(x, xslist, name):
    """ Dibuja las funciones de distribucion acumuladas de las variables

    Args:
        X: vector con la variable del registro SIMAR
        XS: vector con la variable del registro simulado

    Returns:
        Las graficas comparativas de la cdf y pdf

    """

    plt.style.use('ggplot')


    fig, ax1 = plt.subplots(figsize=(4, 4))
    for xs in xslist:
        yp, xp = np.histogram(x, bins=25, normed=True)
        ysp, xsp = np.histogram(xs, bins=25, normed=True)
        xp, xsp = np.diff(xp)/2 + xp[0:-1], np.diff(xsp)/2 + xsp[0:-1]
        plt.plot(xp, yp, 'gray')
        plt.plot(xsp, ysp, 'k')
    plt.xlabel(name, fontweight='bold')
    plt.ylabel('pdf', fontweight='bold')

    fig, ax1 = plt.subplots(figsize=(4, 4))
    for xs in xslist:
        Yp = np.cumsum(yp)/np.cumsum(yp)[-1]
        Yps = np.cumsum(ysp)/np.cumsum(ysp)[-1]
        plt.plot(xp, Yp, 'gray', label='Registro')
        plt.plot(xsp, Yps, 'k', label='Simulacion')
    plt.ylabel('cdf', fontweight='bold')
    # plt.legend(loc='best', numpoints=1)
    plt.gcf().subplots_adjust(bottom=0.2, left=0.15, right=0.85)

    plt.figure(figsize=(4, 4))
    for xs in xslist:
        plt.plot(xp, norm.ppf(Yp), 'gray', label='Registro')
        plt.plot(xsp, norm.ppf(Yps), 'k', label='Simulacion')

    ytick = [.001, .01, .1, .25, .5, .75, .9, .99, .999]
    ytickp = norm.ppf(ytick)
    plt.yticks(ytickp, ytick)
    plt.xlabel(name, fontweight='bold')
    plt.ylabel('Z', fontweight='bold')
    # plt.legend(loc='best', numpoints=1)
    plt.gcf().subplots_adjust(bottom=0.2, left=0.2)

    return


def cdfpdf_anual(n, x, ns, xs, name, ruta):
    """ Dibuja las funciones de distribucion acumuladas anuales registradas y
    simuladas

    Args:
        N: fecha en formato numerico de la senal X
        X: serie registrada
        NS: fecha en formato numerico de la senal XS
        XS: serie simulada

    Returns:
        Dibuja la cdf y pdf anual de las dos series

    """

    plt.style.use('ggplot')
    fig = plt.figure(figsize=(8, 4))
    ndat = 365*24/3.
    nbin = np.round((x.max()-x.min())*ndat**(1./3)/(2.*np.percentile(x, [75, 25])))
    nbin = np.max([int(nbin[1] - nbin[0]), 24])
    for i in range(xs.index.year.min(), xs.index.year.max()+1):
        xaux = xs.index.year == i
        ysp, xsp = np.histogram(xs[xaux], bins=nbin, normed=True)
        xsp = np.diff(xsp)/2 + xsp[0:-1]
        if i == xs.index.year.min():
            plt.subplot(1, 2, 1)
            plt.plot(xsp, ysp, 'k')
        else:
            plt.plot(xsp, ysp, 'k')

    for i in range(x.index.year.min(), x.index.year.max()+1):
        xaux = x.index.year == i
        yp, xp = np.histogram(x[xaux], bins=nbin, normed=True)
        xp = np.diff(xp)/2 + xp[0:-1]
        plt.plot(xp, yp, 'gray')

    plt.xlabel(name, fontweight='bold')
    plt.ylabel('pdf', fontweight='bold')
    plt.gcf().subplots_adjust(bottom=0.2, left=0.2)
    plt.legend(('Simulado', 'Registro'), loc='best')

    for i in range(xs.index.year.min(), xs.index.year.max()+1):
        xaux = xs.index.year == i
        ysp, xsp = np.histogram(xs[xaux], bins=nbin, normed=True)
        xsp = np.diff(xsp)/2 + xsp[0:-1]
        Ysp = np.cumsum(ysp)/np.cumsum(ysp)[-1]

        if i == xs.index.year.min():
            plt.subplot(1, 2, 2)
            plt.plot(xsp, norm.ppf(Ysp), 'k')
        else:
            plt.plot(xsp, norm.ppf(Ysp), 'k')

    for i in range(x.index.year.min(), x.index.year.max()+1):
        xaux = x.index.year == i
        yp, xp = np.histogram(x[xaux], bins=nbin, normed=True)
        xp = np.diff(xp)/2 + xp[0:-1]
        Yp = np.cumsum(yp)/np.cumsum(yp)[-1]
        plt.plot(xp, norm.ppf(Yp), 'gray')

    plt.xlabel(name, fontweight='bold')
    plt.ylabel('Z', fontweight='bold')
    ytick = [.001, .01, .1, .25, .5, .75, .9, .99, .999]
    ytickp = norm.ppf(ytick)
    plt.yticks(ytickp, ytick)
    plt.legend(('Simulado', 'Registro'), loc='best')
    plt.gcf().subplots_adjust(bottom=0.2, left=0.2)

    fig.savefig(ruta + '/cdfpdf_anual_'+name + '.png')


def corr_cruzada_registro(xy, lags, name):
    """ Dibuja la correlacion cruzada entre las variables x e y y xs e ys
    simulada

    Args:
        xy: variables registradas
        xys: variables simuladas
        lags: valor maximo de lags a dibujar
        name: nombre completo de la grafica

    Returns:
        Las graficas

    """

    plt.style.use('ggplot')
    tiempo = np.arange(-lags, lags, 3/24.)
    i1, i2 = len(xy[0])/2.-len(tiempo)/2., len(xy[0])/2.+len(tiempo)/2.
    ccf = np.correlate(xy[0], xy[1], mode='same')
    ccf = ccf/len(ccf) - np.mean(xy[0])*np.mean(xy[1])
    ccf = ccf/(np.std(xy[0])*np.std(xy[1]))

    plt.figure()
    plt.plot(tiempo, ccf[i1:i2], '.', color='gray')
    plt.ylim(ymax=1)

    plt.legend(('Registrado'), loc='best')
    plt.xlabel('Tiempo [dias]')
    plt.ylabel(name)
    plt.gcf().subplots_adjust(bottom=0.2, left=0.2)

    return


def corr_cruzada(xy, xys, lags, name):
    """ Dibuja la correlacion cruzada entre las variables x e y y xs e ys
    simulada

    Args:
        xy: variables registradas
        xys: variables simuladas
        lags: valor maximo de lags a dibujar
        name: nombre completo de la grafica

    Returns:
        Las graficas

    """

    plt.style.use('ggplot')
    tiempo = np.arange(-lags, lags, 3/24.)
    i1, i2 = len(xy[0])/2.-len(tiempo)/2., len(xy[0])/2.+len(tiempo)/2.
    i1s, i2s = len(xys[0])/2.-len(tiempo)/2., len(xys[0])/2.+len(tiempo)/2.
    ccf = np.correlate(xy[0], xy[1], mode='same')
    ccf = ccf/len(ccf) - np.mean(xy[0])*np.mean(xy[1])
    ccf = ccf/(np.std(xy[0])*np.std(xy[1]))
    ccf2 = np.correlate(xys[0], xys[1], mode='same')
    ccf2 = ccf2/len(ccf2) - np.mean(xys[0])*np.mean(xys[1])
    ccf2 = ccf2/(np.std(xys[0])*np.std(xys[1]))

    plt.figure()
    plt.plot(tiempo, ccf[i1:i2], '.', color='gray')
    plt.plot(tiempo, ccf2[i1s:i2s], 'k')
    plt.ylim(ymax=1)

    plt.legend(('Registrado', 'Simulado'), loc='best')
    plt.xlabel('Tiempo [dias]')
    plt.ylabel(name)
    plt.gcf().subplots_adjust(bottom=0.2, left=0.2)

    return


def autocorr(x, xs, lags, var):
    """ Dibuja la autocorrelacion entre una determinada variable registrada y
    simulada

    Args:
        x: vector de la variable registrada
        xs: vector de la variable simulada
        lags: valor maximo de lags a dibujar
        var: nombre de la variable

    Returns:
        Dibuja la grafica

    """

    plt.style.use('ggplot')
    name = 'ACF '+var
    time = np.arange(0, lags)*3./24
#    xtick1 = np.arange(0, time[-1])
    acf = np.correlate(x, x, mode='full')[len(x)-1:]
    acfs = np.correlate(xs, xs, mode='full')[len(xs)-1:]


# figure('Position',[307 312 250 225])
    plt.plot(acf, 'o')
    plt.plot(acfs)
    plt.xlim([0, time[-1]])
#    plt.ylim([0, 1])
    plt.xlabel('Tiempo [dias]', fontsize=8)
    plt.ylabel(name, fontsize=8)

    return


def dens_bivar_reg(x, y, limx, limy, piex, piey):
    """ Dibuja la funcion de densidad bivariada entre las variables x e y

    Args:
        x: serie registrada y simulada de la variable principal
        y: serie registrada y simulada de la variable secundaria
        dx, dy: paso entre variables a dibujar
        limx, limy: limites de la representacion
        piex, piey: texto para los ejes

    Returns:
        La grafica

    """

    plt.style.use('ggplot')

    nn, nx, ny = np.histogram2d(x, y, normed=True)
    nx, ny = np.meshgrid(nx[:-1] + np.diff(nx)/2., ny[:-1] + np.diff(ny)/2.)

    plt.figure(figsize=(8, 4))
    plt.subplot(1, 2, 1)
    plt.contourf(nx, ny, nn)
    plt.colorbar()
    plt.xlim(limx)
    plt.ylim(limy)
    plt.xlabel(piex)
    plt.ylabel(piey)

    return

def dens_bivar(x, y, dx, dy, limx, limy, piex, piey):
    """ Dibuja la funcion de densidad bivariada entre las variables x e y

    Args:
        x: serie registrada de la variable principal
        y: serie registrada de la variable secundaria
        dx, dy: paso entre variables a dibujar
        limx, limy: limites de la representacion
        piex, piey: texto para los ejes

    Returns:
        La grafica

    """

    plt.style.use('ggplot')

    nn, nx, ny = np.histogram2d(x[0], y[0], normed=True)
    nx, ny = np.meshgrid(nx[:-1] + np.diff(nx)/2., ny[:-1] + np.diff(ny)/2.)

    plt.figure(figsize=(8, 4))
    plt.subplot(1, 2, 1)
    plt.contourf(nx, ny, nn)
    plt.xlim(limx)
    plt.ylim(limy)
    plt.xlabel(piex)
    plt.ylabel(piey)

    nn, nx, ny = np.histogram2d(x[1], y[1], normed=True)
    nx, ny = np.meshgrid(nx[:-1] + np.diff(nx)/2., ny[:-1] + np.diff(ny)/2.)
    plt.subplot(1, 2, 2)
    plt.contourf(nx, ny, nn, lw=1)
    plt.xlim(limx)
    plt.ylim(limy)
    plt.xlabel(piex)
    plt.ylabel(piey)

    return



def series_simuladas(df):
    """ Funcion que dibuja las series completas de las variables
    """
    plt.style.use('ggplot')

    lw = len(df.columns)
    plt.figure()
    for i, j in enumerate(df):
        plt.subplot(lw, 1, i+1)
        plt.plot(df[j])
        plt.ylabel(df.columns[i])
        if j != lw:
            plt.xticks(visible=False)
        else:
            plt.xticks(visible=True)

        if ((j == 'dh') | (j=='dv')):
                plt.ylim([0, 360])
                plt.yticks([0, 90, 180, 270, 360])

    plt.xticks(rotation=30)


def param_no_est(par, mod, name):
    plt.style.use('ggplot')

    n = np.linspace(0, 1, 100)
    cossen = np.zeros([np.max(mod)*2, len(n)])
    w = 2*np.pi
    for i in range(0, np.max(mod)):
        aux = (i+1)*w*n.T
        cossen[2*(i-1), :] = np.cos(aux)
        cossen[2*(i-1)+1, :] = np.sin(aux)

    if name == 'direc':
        cont = 1
        mu, sig = np.zeros([len(n), 4]), np.zeros([len(n), 4])
        mu[:, 0] = par[0] + np.dot(par[1:2*mod[0]+1], cossen[0:mod[0]*2, :])
        sig[:, 0] = par[2*mod[0]+1] + np.dot(par[2*mod[0]+2:4*mod[0]+2], cossen[0:mod[0]*2, :])
        mu[:, 1] = par[4*mod[0]+2] + np.dot(par[4*mod[0]+3:4*mod[0]+2*mod[1]+3], cossen[0:mod[1]*2, :])
        sig[:, 1] = par[4*mod[0]+2*mod[1]+3] + np.dot(par[4*mod[0]+2*mod[1]+4:4*np.sum(
            mod[0:2])+4], cossen[0:mod[1]*2, :])
        mu[:, 2] = par[4*np.sum(mod[0:2])+4] + np.dot(par[4*np.sum(mod[0:2])+5:4*np.sum(mod[0:2])+2*mod[2]+5], cossen[
            0:mod[2]*2, :])
        sig[:, 2] = par[4*np.sum(mod[0:2])+2*mod[2]+5] + np.dot(par[4*np.sum(mod[0:2])+2*mod[2]+6:4*np.sum(
            mod[0:3])+6], cossen[0:mod[2]*2, :])
        mu[:, 3] = par[4*np.sum(mod[0:3])+6] + np.dot(par[4*np.sum(mod[0:3])+7:4*np.sum(mod[0:3])+2*mod[3]+7], cossen[
            0:mod[3]*2, :])
        sig[:, 3] = par[4*np.sum(mod[0:3])+2*mod[3]+7] + np.dot(par[4*np.sum(mod[0:3])+2*mod[3]+8:4*np.sum(
            mod)+8], cossen[0:mod[3]*2, :])

        plt.figure()
        plt.subplot(2, 2, 1)
        plt.plot(n, mu[:, 0])
        plt.plot(n, sig[:, 0])
        plt.xlabel('N', fontweight='bold')
        plt.legend((r'$\mu_1$', r'$\sigma_1$'), loc='best')

        plt.subplot(2, 2, 2)
        plt.plot(n, mu[:, 1])
        plt.plot(n, sig[:, 1])
        plt.xlabel('N', fontweight='bold')
        plt.legend((r'$\mu_2$', r'$\sigma_2$'), loc='best')

        plt.subplot(2, 2, 3)
        plt.plot(n, mu[:, 2])
        plt.plot(n, sig[:, 2])
        plt.xlabel('N', fontweight='bold')
        plt.legend((r'$\mu_3$', r'$\sigma_3$'), loc='best')

        plt.subplot(2, 2, 4)
        plt.plot(n, mu[:, 3])
        plt.plot(n, sig[:, 3])
        plt.xlabel('N', fontweight='bold')
        plt.legend((r'$\mu_4$', r'$\sigma_4$'), loc='best')
    else:
        cont = 1
        param = np.zeros([3, len(n)])
        for i in range(0, 3):
            param[i, :] = par[i+np.sum(mod[0:i])*2] + np.dot(par[cont:np.sum(mod[0:i+1])*2+1], cossen[0:mod[i]*2, :])
            cont += np.sum(mod[i])*2

        plt.figure()
        plt.plot(n, param[0, :])
        plt.plot(n, param[1, :])
        plt.plot(n, param[2, :])
        plt.xlabel('N', fontweight='bold')
        plt.legend((r'$\mu$', r'$\sigma$', r'$\xi$'), loc='best')


def nmarea_res(nameu, fu, tideconout, mm, mm_min, ma_mean, n):
    """
    """

    daten = dates.num2date(n)
    fig, ax = plt.subplots(figsize=(8, 6))
    n = np.arange(n[0], n[-1], 1/24.)
    daten2 = np.array(dates.num2date(n), dtype=np.float64)
    mast = t_predic(n, nameu, fu, tideconout, lat=36)

    col_labels = ['amp', 'amp_err', 'pha', 'pha_err']
    ind = np.argsort(tideconout[:, 0])
    table_vals = tideconout[ind[-1:-17:-1], :]
    plt.table(cellText=table_vals, colLabels=col_labels, rowLabels=nameu[ind[-1:-17:-1]], loc='center right')
    fig.patch.set_visible(False)
    ax.axis('off')
    plt.title('Componentes armonicas', fontweight='bold')

    plt.style.use('ggplot')
    plt.figure()
    plt.subplot(2, 1, 1)
    plt.plot(daten2, (mast+ma_mean), 'k')
    plt.ylabel('Marea astronomica (m)')
    plt.xticks(visible=False)
    plt.subplot(2, 1, 2)
    plt.plot(daten, (mm+mm_min), color='gray')
    plt.ylabel('Residuo meteorologico (m)')
    plt.xticks(rotation=20)


def cdf_pdf_ajuste_est(x, par, mod, name, fun, funm):
    """
    """

    plt.style.use('ggplot')

    ysp, xsp = np.histogram(x, bins=100, normed=True)
    xsp = np.diff(xsp)/2 + xsp[0:-1]
    xq = np.linspace(np.min(x), np.max(x), 100)
    if ((name == 'dh') | (name == 'dv')):
        lcdf = fun.cdf(xq*0, xq, par[0], mod, 3)
        lmcdf = funm.cdf(xq*0, xq, par[1], [0, 0, 0, 0], 4)
    else:
        lcdf = fun.cdf(xq, *par[0])
        lmcdf = funm.cdf(xq*0, xq, par[1], mod)

    plt.figure(figsize=(4, 4))
    plt.plot(xsp, ysp, '.', color='gray', label='Registro')
    if ((name == 'dh') | (name == 'dv')):
        plt.plot(xq[:-1], np.diff(lcdf)/(xq[1]-xq[0]), 'k', label='3 normenr')
        plt.plot(xq[:-1], np.diff(lmcdf)/(xq[1]-xq[0]), 'k--', label='4 normenr')
    else:
        plt.plot(xq[:-1], np.diff(lcdf)/(xq[1]-xq[0]), 'k', label='Modo simple')
        plt.plot(xq[:-1], np.diff(lmcdf)/(xq[1]-xq[0]), 'k--', label='Modo mixto')
    plt.xlabel(name, fontweight='bold')
    plt.ylabel('Funcion de densidad', fontweight='bold')
    plt.xlim([np.min(xq), np.max(xq)])
    plt.legend(numpoints=1, loc='best')
    plt.gcf().subplots_adjust(bottom=0.2, left=0.2)
#    plt.savefig('pdf')

    plt.figure(figsize=(4, 4))
    plt.plot(xsp, np.cumsum(ysp)*np.diff(xsp)[0], '.', color='gray', label='Registro')
    if ((name == 'dh') | (name == 'dv')):
        plt.plot(xq, lcdf, 'k', label='3 normenr')
        plt.plot(xq, lmcdf, 'k--', label='4 normenr')
    else:
        plt.plot(xq, lcdf, 'k', label='Modo simple')
        plt.plot(xq, lmcdf, 'k--', label='Modo mixto')
    plt.xlabel(name, fontweight='bold')
    plt.ylabel('Funcion de distribucion', fontweight='bold')
    plt.xlim([np.min(xq), np.max(xq)])
    plt.ylim([0, 1])
    plt.legend(numpoints=1, loc='best')
    plt.gcf().subplots_adjust(bottom=0.2, left=0.2)
#    plt.savefig('cdf')
    plt.pause(3)


def cuantiles_ne(nd, xd, *args):
    """ Funcion que dibuja la variacion de determinados cuantiles del registro y del ajuste teorico a lo largo del año

    Args:
        - nd: fecha en el rango [0 - 1]
        - xd: serie de la variable a analizar
        - par: parametros de la funcion de probabilidad
        - mod: orden del desarrollo de fourier
        - fun: funcion de probabilidad

    Returns:
        Grafica con los cuantiles empiricos y teoricos

    """

    units = {'hs': '[m]', 'tp': '[s]', 'dh': '[rad]', 'vv': '[m/s]', 'dv': '[rad]', 'mme': '[m]'}

    dt = 0.001
    taux = np.arange(dt/2, 1-dt/2, dt)
    nt = len(taux)
    t, dp, pemp, _, _ = emp(xd, nd-np.floor(nd))
    xaux = np.zeros(nt)
    fig = plt.figure()
    for i in range(len(pemp)):
        if len(args) == 5:
            par, mod, fun, vars_, nnorm = args
            if ((np.sum(mod)*2+3 == len(par)) | (np.sum(mod)*2+2 == len(par))):
                xaux = ajs.inv(taux, np.ones(nt)*pemp[i], par, mod, fun)
            elif ((vars_ != 'dh') & (vars_ != 'dv')):
                xaux = fun.inv(taux, np.ones(nt)*pemp[i], par, mod)
            elif (((vars_ == 'dh') | (vars_ == 'dv')) & isinstance(nnorm, bool)):
                xaux = fun.inv(taux, np.ones(nt)*pemp[i], par, mod)
            else:
                if i == 0:
                    xaux, cdf_ = fun.inv(taux, np.ones(nt)*pemp[i], par, mod, nnorm)
                else:
                    xaux, _ = fun.inv(taux, np.ones(nt)*pemp[i], par, mod, nnorm, cdf_)
            plt.plot(taux, xaux, 'k', lw=2)
        else:
            vars_ = args[0]
        plt.plot(t, dp[:, i], 'k')

        if len(args) == 5:
            plt.legend(['Ajuste', 'Empirico'], loc='best')
            plt.xlabel('N (0 - 1)')
            plt.ylabel(vars_ + units[vars_])
        else:
            plt.legend(['Empirico'], loc='best')
            plt.xlabel('N (0 - 1)')
            plt.ylabel(vars_ + units[vars_])

    if len(args) == 5:
        fig.savefig('cuantiles_ne_' + vars_ + '.png')
    else:
        fig.savefig('cuantiles_ne_' + vars_ + '.png')
    return


def emp(x, nd):
    """ Funcion que calcula el valor de los cuantiles
    """

    t, vt = np.arange(0., 1., 0.002), 1/24
    nt = len(t)

    pemp = np.array([.01, .05, .1, .25, .5, .75, .9, .95, .99, .995, .999])
    xp, m, d = np.zeros([nt, len(pemp)]), np.zeros([nt, 1]), np.zeros([nt, 1])
    for i in range(nt):
        tinf, tsup = t[i] - vt, t[i] + vt
#        if tinf < 0:
#            tinf += 1
#            id_ = np.where((n > tinf) | (n < tsup))[0]
#        elif tsup > 1:
#            tsup -= 1
#            id_ = np.where((n > tinf) | (n < tsup))[0]
#        else:
        id_ = np.where((nd > tinf) & (nd < tsup))[0]

        xaux = np.sort(x.iloc[id_])
        naux = len(xaux)
        paux = np.arange(1, naux+1)/(naux+1)
        for j in range(len(pemp)):
            if xaux is None:
                xp[i, j] = 0
            else:
                xp[i, j] = np.interp(pemp[j], paux, xaux)
        m[i], d[i] = np.mean(xaux), np.std(xaux)
    return t, xp, pemp, m, d


def rosa_oleaje_viento(df):
    title_wave = 'Rosa Oleaje'
    title_wind = 'Rosa Viento'
    legend_wave = 'upper left'
    legend_wind = 'upper left'

    fig = plt.figure(figsize=(10, 10))

    ax1 = fig.add_subplot(121, projection='windrose')
    ax1.bar(df['dh'], df['hs'], normed=True, opening=0.8, edgecolor='white')
    ax1.set_title(title_wave, y=1.08)
    ax1.legend(loc=legend_wave, fontsize='small')

    ax2 = fig.add_subplot(122, projection='windrose')
    ax2.bar(df['dv'], df['vv'], normed=True, opening=0.8, edgecolor='white')
    ax2.set_title(title_wind, y=1.08)
    ax2.legend(loc=legend_wind, fontsize='small')

    return

def espec(x, name, ud, dt, k=10):

    fig = plt.figure()

    Sxx, f = FDS(x, dt, k=10)
    plt.loglog(f, Sxx)
    plt.ylim(ymin=10 ** -1)
    plt.xlabel(r'$f [Hz]$')
    plt.ylabel(r'$S(f)$ $[' + ud + '^2s]$')
    fig.suptitle(name)

def dispersion(df, var_ppal):

    fig = plt.figure(figsize=(10, 5))

    df2 = df.drop([var_ppal], 1)
    nvars = len(df2.columns)

    name_df = ['hs', 'tp', 'dh', 'vv', 'dv', 'pa', 'niv', 'mas', 'mme', 'tm', 'per']
    name_ylabel = ['$H_s (m)$', '$T_p (s)$', r'$\theta(^\circ)$', '$U (m/s)$', r'$\theta_U(^\circ)$', '$P_s (Hpa)$',
                   r'$\eta_{NM} (m)$', r'$\eta_{MAs} (m)$', r'$\eta_{MMe} (m)$', '$T_{m02} (s)$', '$H_s/L$']

    ind_xlb = name_df.index(var_ppal)
    for j in range(nvars):
        plt.subplot(1, nvars, j + 1)
        plt.plot(df[var_ppal].values, df2.iloc[:, j].values, '.')
        ind_ylb = name_df.index(df2.iloc[:, j].name)
        plt.ylabel(name_ylabel[ind_ylb])
        plt.xlabel(name_ylabel[ind_xlb])

    plt.tight_layout(pad=0.2, w_pad=0.1, h_pad=0.4)

