# -*- coding: utf-8 -*-
import matplotlib.pyplot as plt
import numpy as np
from scipy.stats import expon, poisson, norm


def plot_d_n_delta(dur, n, delta_d):
    """ Funcion que dibuja los ajustes probabilisticos de la duracion de los ciclos, el numero de ciclos por año y
    desfase entre picos de las variables principales.

    Args:
        dur: duracion de cada ciclo
        n: numero de ciclos
        delta_d: desfase adimensional entre los picos de las variables principales

    Returns:

    """

    plt.style.use('ggplot')
    plt.figure()
    plt.subplot(1, 3, 1)
    hist, edg = np.histogram(dur, normed=True)
    plt.plot(edg[:-1] + np.diff(edg), hist)
    par = expon.fit(dur)
    x = np.linspace(edg[0], edg[-1], 100)
    plt.plot(x, expon.pdf(x, *par))

    plt.subplot(1, 3, 2)
    hist, edg = np.histogram(n, normed=True)
    plt.plot(edg[:-1] + np.diff(edg), hist)
    par = poisson.fit(n)
    x = np.linspace(edg[0], edg[-1], 100)
    plt.plot(x, poisson.pdf(x, *par))

    plt.subplot(1, 3, 3)
    hist, edg = np.histogram(delta_d, normed=True)
    plt.plot(edg[:-1] + np.diff(edg), hist)
    par = norm.fit(delta_d)
    x = np.linspace(edg[0], edg[-1], 100)
    plt.plot(x, norm.pdf(x, *par))
