# -*- coding: utf-8 -*-
"""
Created on Mon Jun 13 23:04:31 2016

@author: m_cob
"""
from __future__ import division
import matplotlib.pyplot as plt
import numpy as np
import scipy as sc


def plot_pot_lmom(umb, treval, valmed_lmom, liminf, limsup, au2_lmom, au2pv_lmom, nanios, nvar):
    """ Funcion que dibuja los resultados del analisis de picos sobre umbral con el metodos de los momentos.
    """
    plt.style.use('ggplot')
    xlim = (np.floor(np.min(umb)), np.ceil(np.max(umb)))
    fig = plt.figure(figsize=(16, 10))
    fig.subplots_adjust(left=0.2, wspace=0.4)

    # MRLP
    ax1 = plt.subplot2grid((3, 3), (0, 0))
    plt.plot(umb, valmed_lmom[:, 3], 'k-', lw=2)
    plt.plot(umb, limsup[:, 3], 'k--', umb, liminf[:, 3], 'k--')
    plt.ylabel('MRLP')
    plt.grid(True)
    plt.xlim(xlim)
    ax1.get_xaxis().set_ticklabels([])
    ax1.yaxis.set_label_coords(-0.2, 0.5)

    # Forma
    ax2 = plt.subplot2grid((3, 3), (1, 0))
    plt.plot(umb, valmed_lmom[:, 0], 'k-', lw=2)
    plt.plot(umb, limsup[:, 0], 'k--', umb, liminf[:, 0], 'k--')
    plt.ylabel('GP forma k')
    plt.grid(True)
    plt.xlim(xlim)
    ax2.get_xaxis().set_ticklabels([])
    ax2.yaxis.set_label_coords(-0.2, 0.5)

    # Escala modif
    ax3 = plt.subplot2grid((3, 3), (2, 0))
    plt.plot(umb, valmed_lmom[:, 4], 'k-', lw=2)
    plt.plot(umb, limsup[:, 4], 'k--', umb, liminf[:, 4], 'k--')
    plt.ylabel('Modificado GP escala $\sigma^*$')
    plt.grid(True)
    plt.xlabel('Umbral')
    plt.xlim(xlim)
    ax3.yaxis.set_label_coords(-0.2, 0.5)

    # Gráficas QTR
    for i in range(0, np.size(treval)):
        ax = plt.subplot2grid((3, 3), (0+i, 1))
        plt.plot(umb, valmed_lmom[:, 6+i], 'k-', lw=2)
        plt.hold(True)
        plt.plot(umb, limsup[:, 6+i], 'k--', umb, liminf[:, 6+i], 'k--')
        plt.ylabel(nvar + r'$ - TR\quad '+str(int(treval[i]))+' a$'.decode('utf-8'))
        plt.grid(True)
        plt.xlim(xlim)
        plt.ylim((np.floor(np.min(liminf[:, 6+i])), np.ceil(np.max(limsup[:, 6+i]))))
        if i < np.size(treval):
            ax.get_xaxis().set_ticklabels([])
    
    plt.xlabel('Umbral')

    # Gráficas Estadístico y p-valor
    ax7 = plt.subplot2grid((3, 3), (0, 2), rowspan=2)
    plt.plot(umb, au2_lmom, 'k-', lw=2)
    plt.ylabel('estadistico $A_R^2$'.decode('utf-8'))
    ax7.yaxis.set_label_coords(-0.2, 0.5)
    ax7.get_xaxis().set_ticklabels([])
    ax8 = ax7.twinx()
    plt.plot(umb, au2pv_lmom, 'k--')
    ax8.set_ylabel('$1-p_{value}$')
    plt.grid(True)
    plt.xlim(xlim)

    # Eventos/año
    ax9 = plt.subplot2grid((3, 3), (2, 2))
    plt.plot(umb, valmed_lmom[:, 5], 'k-', lw=2)
    plt.ylabel('Picos anuales'.decode('utf-8'))
    plt.xlabel('Umbral')
    plt.grid(True)
    plt.xlim(xlim)
    ax9.yaxis.set_label_coords(-0.2, 0.5)
    ax10 = ax9.twinx()
    nl = len(ax9.get_yticks())
    nmax = np.ceil(np.ceil(np.max(valmed_lmom[:, 5]))*nanios/(nl-1))
    n2l = np.linspace(0, nmax*(nl-1), nl)

    plt.ylabel('Numero total de picos'.decode('utf-8'))
    ax10.set_ylim([0, np.ceil(np.max(valmed_lmom[:, 5]))*nanios])
    plt.xlim(xlim)
    plt.yticks(n2l)


def plot_autom(boot, orig, ci, tr, picos, npicos, eventanu, emax, eagente):
    """ Funcion que dibuja los resultados del analisis de picos sobre umbral a partir de tecnicas de remuestreo.
    """
    plt.style.use('ggplot')
    fig = plt.figure(figsize=(12, 8))
    fig.subplots_adjust(left=0.2, wspace=0.4)

    # Histograma de k
    ax1 = plt.subplot2grid((2, 3), (0, 0))
    n0, c0 = np.histogram(boot[0][0][:, 0], bins=20)
    n0 = n0/np.sum(n0)/(c0[1]-c0[0])
    plt.bar(c0[:-1]+(c0[1]-c0[0])/2, n0, width=c0[1]-c0[0], color='gray', lw=0)
    if boot[1]:
        plt.plot(np.array([0, 0]), ax1.get_ylim(), 'r--')

    plt.plot(orig[0][0][0]*np.array([1, 1]), ax1.get_ylim(), 'b--')
    plt.xlabel(emax+' forma k')
    plt.ylabel('Frecuencia')
    ax1.get_yaxis().set_ticks([])

    # Histograma de sigma
    ax2 = plt.subplot2grid((2, 3), (0, 1))
    n0, c0 = np.histogram(boot[0][0][:, 2], bins=20)
    n0 = n0/np.sum(n0)/(c0[1]-c0[0])
    plt.bar(c0[:-1]+(c0[1]-c0[0])/2, n0, width=c0[1]-c0[0], color='gray', lw=0)

    if boot[1]:
        n1, c1 = np.histogram(boot[1][0][:, 2], bins=20)
        n1 = n1/np.sum(n1)/(c1[1]-c1[0])
        plt.step(c1[:-1], n1, 'r', where='post')
        plt.plot(orig[1][0][2]*np.array([1, 1]),  ax2.get_ylim(), 'r--')

    plt.plot(orig[0][0][2]*np.array([1, 1]), ax2.get_ylim(), 'b--')
    plt.xlabel(emax+' escala $\sigma$')
    plt.ylabel('Frecuencia')
    ax2.get_yaxis().set_ticks([])

    # Histograma de mu
    ax3 = plt.subplot2grid((2, 3), (0, 2))
    n0, c0 = np.histogram(boot[0][0][:, 1], bins=20)
    n0 = n0/np.sum(n0)/(c0[1]-c0[0])
    plt.bar(c0[:-1]+(c0[1]-c0[0])/2, n0, width=c0[1]-c0[0], color='gray', lw=0)
    if boot[1]:
        n1, c1 = np.histogram(boot[1][0][:, 1], bins=20)
        n1 = n1/np.sum(n1)/(c1[1]-c1[0])
        plt.step(c1[:-1], n1, 'r', where='post')
        plt.plot(orig[1][0][1]*np.array([1, 1]), ax3.get_ylim(), 'r--')

    plt.plot(orig[0][0][1]*np.array([1, 1]), ax3.get_ylim(), 'b--')
    plt.xlabel(emax+' posicion $\mu$'.decode('utf-8'))
    plt.ylabel('Frecuencia')
    ax3.get_yaxis().set_ticks([])

    # Histograma de Tr 1000
    ax4 = plt.subplot2grid((2, 3), (1, 2))
    n0, c0 = np.histogram(boot[0][0][:, -1], bins=20)
    n0 = n0/np.sum(n0)/(c0[1]-c0[0])
    plt.bar(c0[:-1]+(c0[1]-c0[0])/2, n0, width=c0[1]-c0[0], color='gray', lw=0)
    plt.plot(orig[0][0][-1]*np.array([1, 1]), ax4.get_ylim(), 'b--')
    plt.xlabel(eagente + '$_{Tr\quad 1000\quad yy}$'.decode('utf-8'))
    plt.ylabel('Frecuencia')
    ax4.get_yaxis().set_ticks([])
    if boot[1]:
        n1, c1 = np.histogram(boot[1][0][:, -1], bins=20)
        n1 = n1/np.sum(n1)/(c1[1]-c1[0])
        plt.step(c1[:-1], n1, 'r', where='post')
        plt.plot(orig[1][0][-1]*np.array([1, 1]), ax4.get_ylim(), 'r--')

    # Gráfico Tr-Xtr
    ax5 = plt.subplot2grid((2, 3), (1, 0), colspan=2)
    plt.fill(np.hstack((tr, tr[::-1])), np.hstack((ci[0][0][0, 4:], ci[0][0][1, ::-1][:-4])), color='gray', lw=0)
    if emax == 'GP':
        plt.plot(1./((1.-np.arange(1., npicos+1.)/(npicos+1.))*eventanu), np.sort(picos), 'g.', markersize=8)
    elif emax == 'GEV':
        plt.plot(1./npicos[::-1], np.sort(eventanu), 'g.', markersize=8)

    plt.plot(tr, orig[0][0][4:], 'b--')
    if boot[1]:
        plt.plot(tr, orig[1][0][4:], 'r--')
        plt.plot(tr, ci[1][0][0, 4:], 'r-.')
        plt.plot(tr, ci[1][0][1, 4:], 'r-.')

    plt.xlim([1, 1000])
    ax5.set_xscale('log', nonposx='clip')
    plt.ylabel(eagente)
    plt.xlabel('Periodo de retorno (years)'.decode('utf-8'))


def plot_serie_picos(df_serie, df_picos, ylab, nombre):
    """ Funcion que dibuja la serie temporal y picos de la variable
    """

    plt.style.use('ggplot')
    plt.figure()
    plt.plot(df_serie.index, df_serie.iloc[:, 0], color='b')
    plt.plot(df_picos.index, df_picos.iloc[:, 0], 'ro', color='orange')
    plt.ylabel(ylab)
    plt.xticks(rotation=30)
    plt.legend(('Serie temporal', 'Picos'), loc='best')
    plt.title(nombre)


def plot_serie_picos_umbral(df_serie, df_picos, ylab, umbral, nombre):
    """ Funcion que dibuja la serie temporal y los valores sobre umbral de la variable
    """

    plt.style.use('ggplot')
    plt.figure()
    plt.plot(df_serie.index, df_serie.iloc[:, 0], color='b')
    plt.plot(df_picos.index, df_picos.iloc[:, 0], 'ro', color='orange')
    plt.ylabel(ylab)
    plt.xticks(rotation=30)
    plt.legend(('Serie temporal', 'Picos (umbral = {:.2f}' .format(umbral) + ')'), loc='best')

    plt.title(nombre)
