# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
from wrap_to_2pi import wrap_to_2pi


def normalizar(x, v):
    """Funcion para normalizar variables respecto a la distancia maxima
    entre sus valores.

    * Si v == 0 (variable escalar)
      xnorm = (x-xmin)/(xmax-xmin)
    * Si v == 1 (variable circular en grados sexagesimales)
      xnorm = x/pi

    Args:
        x: Vector de valores.
        v: Naturaleza de la variable (0 variable escalar, 1 variable circular).

    Returns:
        Variable normalizada.

    """

    xnorm = np.zeros(0)
    if v == 0:
        xmax = max(x)
        xmin = min(x)
        xnorm = (x - xmin) / (xmax - xmin)
    elif v == 1:
        x = wrap_to_2pi(np.deg2rad(x))
        xnorm = x / np.pi
    
    return xnorm
