# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
from dist_cnorm import dist_cnorm
import radial_basis_functions
import matplotlib.pyplot as plt


def reconstruction_agente(x, d_j, n, casos_propagados, nvars, c_i, c_f, n_c, esc, circ):
    data = n
    vc_vars = list()
    e_vars = list()
    posmin_vars = list()
    for ag in range(nvars):
        v_pj = casos_propagados[ag, :]
        vc = np.linspace(c_i, c_f, n_c)
        e = np.zeros(vc.shape)

        for j, i in enumerate(vc):
            (_, _, e[j]) = radial_basis_functions.radial_basis_functions(d_j, v_pj, esc, circ, i)

        posmin = np.argmin(e)
        c = vc[posmin]

        vc_vars.append(vc)
        e_vars.append(e)
        posmin_vars.append(posmin)

        a, b, _ = radial_basis_functions.radial_basis_functions(d_j, v_pj, esc, circ, c)
        x_p = reconstruction_rbf(x, d_j, a, b, c, esc, circ)
        data = np.vstack((data, x_p))

    return data, vc_vars, e_vars, posmin_vars


def plot_minim_c(vc_vars, e_vars, posmin_vars):

    plt.style.use('ggplot')
    plt.figure()

    name_ylabel = ['$E_{H_s}$', '$E_{T_z}$', r'$E_{\theta}$']

    nvars = len(vc_vars)
    for j in range(nvars):
        plt.subplot(nvars, 1, j + 1)
        plt.plot(vc_vars[j], e_vars[j])
        plt.plot(vc_vars[j][posmin_vars[j]], e_vars[j][posmin_vars[j]], 'ro', color='orange')
        plt.ylabel(name_ylabel[j])
        if j < len(vc_vars)-1:
            plt.xticks(visible=False)
        else:
            plt.xticks(visible=True)


def reconstruction_rbf(x, d_j, a, b, c, esc, circ):
    """Reconstruccion de la serie temporal a traves de las funciones de base radial

    Args:
        x: Matriz de datos de entrada (n datos x nvar variables).
        d_j: Casos representativos de las condiciones de entrada (matriz de m casos x nvar variables).
        a: Coeficientes de ajuste de las funciones gaussianas.
        b: Coeficientes de ajuste de las funciones lineales.
        c: Coeficiente de forma de la funcion gaussiana.
        esc: Indices de las variables escalares en columnas de las las matrices ``x`` y ``d_j``.
        circ: Indices de las variables circulares en columnas de las las matrices ``x`` y ``d_j``.

    Returns:
        Serie temporal reconstruida.
       
    """

    n = x.shape[0]
    m = a.shape[0]

    phi_ij = np.zeros((n, m))
    for j in range(n):
        for k in range(m):
            ds2, dc2 = 0, 0
            for l in range(esc.size):
                ds2 += (x[j, esc[l]] - d_j[k, esc[l]]) ** 2
            for l in range(circ.size):
                dc2 += (dist_cnorm(x[j, circ[l]], d_j[k, circ[l]])) ** 2
            dist2 = ds2 + dc2
            phi_ij[j, k] = np.exp(-dist2 / (2 * (c ** 2)))

    m_ij = np.hstack((np.ones((n, 1)), x))

    x_p = np.dot(phi_ij, a) + np.dot(m_ij, b)

    return x_p


