# -*- coding: utf-8 -*-
from __future__ import division
import matplotlib.pyplot as plt
import numpy as np


def periodo_retorno(t, uinc, amed, t_emp, x, ylab):
    """ Funcion que dibuja los periodo de retorno para la variable ``x``

    Args:
        - t: serie con los periodos de retorno
        - x: serie con la variable principal
        - uinc: umbral de incertimbre
        - amed: ajuste medio
        - ylab: etiqueta con el nombre de la variable

    Returns:
        La grafica de todas las series

    """

    plt.figure()
    plt.semilogx(t_emp, x, '.', color='gray')
    plt.semilogx(t, amed - uinc, '--k')
    plt.semilogx(t, amed, 'k', lw=2)
    plt.semilogx(t, amed + uinc, '--k')
    plt.grid(True)
    plt.xlabel('Periodo de retorno (anios)')
    plt.ylabel(ylab)


def sel_estadisticos(h, nvardep, ndir, *args):
    """ Funcion que dibuja la seleccion de estadisticos de estados compatibles

    Args:
        - h: variable independiente
        - nvardep: numero de variables dependientes
        - ndir: numero de direcciones principales
        - *args: tupla con contiene en orden: las variables dependientes para cada valor de la variable independiente;
            el valor del ajuste y los nombres de todas las variables.

    """

    plt.figure()
    symb = ['<', 's', 'o', 'd']
    col = [['brown', 'orange'], ['blue', 'purple'], ['k', 'gray'], ['green', 'yellow'], ['brown', 'yellow']]
    for i in range(0, nvardep):
        for j in range(0, ndir):
            plt.subplot(2, int(np.ceil(nvardep/2)), i+1)
            plt.plot(h, args[ndir*i+j], marker=symb[j], color=col[i, j])
            plt.plot(h, args[ndir*nvardep+ndir*i+j], '--', color=col[i, j])
        plt.xlabel(args[2*ndir*nvardep+ndir*i+j])
        plt.ylabel(args[2*ndir*nvardep+ndir*i+j+1])


def verificacion(diec, xlab, ylab):
    """ Funcion que dibuja las pdfs de cada uno de los margenes de seguridad y el conjunto

    Args:
        - diec: coeficientes beta del margen de seguridad por estados en vida util de deslizamiento,
            inestabilidad de la berma, erosion y conjunto
        - xlab: etiquetas del eje x
        - ylab: etiquetas del eje y

    Returns:
        Devuelve las graficas de las pdfs de todos ellos

    """

    fig = plt.figure()
    fig.subplots_adjust(hspace=0.5)

    plt.subplot2grid((3,3), (0,0))
    diec_temp = diec[:, 0]
    diec_temp = diec_temp[~np.isnan(diec_temp)]
    diec_temp = diec_temp[~np.isinf(diec_temp)]
    plt.hist(diec_temp, bins=15, normed=True, color='orange')
    plt.xlabel(xlab[0])
    plt.ylabel(ylab[0])

    plt.subplot2grid((3,3), (1,0))
    diec_temp = diec[:, 1]
    diec_temp = diec_temp[~np.isnan(diec_temp)]
    diec_temp = diec_temp[~np.isinf(diec_temp)]
    plt.hist(diec_temp, bins=15, normed=True, color='orange')
    plt.xlabel(xlab[1])
    plt.ylabel(ylab[1])

    plt.subplot2grid((3,3), (2,0))
    diec_temp = diec[:, 2]
    diec_temp = diec_temp[~np.isnan(diec_temp)]
    diec_temp = diec_temp[~np.isinf(diec_temp)]
    plt.hist(diec_temp, bins=15, normed=True, color='orange')
    plt.xlabel(xlab[2])
    plt.ylabel(ylab[2])

    plt.subplot2grid((3, 3), (0, 1), colspan=2, rowspan=3)
    diec_temp = diec[:, 3]
    diec_temp = diec_temp[~np.isnan(diec_temp)]
    diec_temp = diec_temp[~np.isinf(diec_temp)]
    plt.hist(diec_temp, bins=15, normed=True, color='orange')
    plt.xlabel(xlab[3])
    plt.ylabel(ylab[3])

